@@ -11,11 +11,15 @@ class ApoJWT():
     iss: Issuer string used for additional security
     admin_audience: The name of the audience that has access to all endpoints. Default admin
     algorithm: The algorithm to use when encoding/decoding. Default HS256
-    token_finder: Function used to retrive the JWT from the http request. Default None
+    token_finder: Zero Parameter function used to retrive the JWT from the http request. Default None
+    permission_formatter: One Parameter (permission name) function used to format the permissions passed to permission_required
+        Can be used to include request data in the permission name
+        Example - lambda permission_name: f"{permission_name}:request.json['trial_id']"
     """
 
-    def __init__(self, secret: str, iss: str="", admin_audience: str="admin", algorithm: str="HS256", token_finder=None):
+    def __init__(self, secret: str, iss: str="", admin_audience: str="admin", algorithm: str="HS256", token_finder=None, permission_formatter=None):
         self.token_f = token_finder
+        self.permission_formatter = permission_formatter
         self.secret = str(secret)
         self.iss = str(iss)
         self.algo = algorithm
@@ -61,8 +65,9 @@ class ApoJWT():
                 if self.token_f is None:
                     raise TypeError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
                 token = self.token_f()
+                formatted_perm = permission if self.permission_formatter is None else self.permission_formatter(permission)
                 try:
-                    jwt.decode(token, self.secret, issuer=self.iss, audience=permission, algorithms=[self.algo])
+                    jwt.decode(token, self.secret, issuer=self.iss, audience=formatted_perm, algorithms=[self.algo])
                 except jwt.exceptions.InvalidAudienceError:
                     jwt.decode(token, self.secret, issuer=self.iss, audience=self.admin_aud, algorithms=[self.algo])
                 return fn(*args, **kwargs)
